
#include <CoreServices/CoreServices.h> 
#include <AudioUnit/AudioUnit.h>
#include <AudioToolbox/AudioToolbox.h> 
#include <unistd.h>

// $(ARCHS_STANDARD_32_BIT)

// This call creates the Graph and the Synth unit...
static OSStatus	CreateAUGraph (AUGraph *outGraph, AudioUnit *outSynth)
{
  OSStatus result;
  //create the nodes of the graph
    AUNode synthNode, limiterNode, outNode;
	
  AudioComponentDescription cd;
  cd.componentManufacturer = kAudioUnitManufacturer_Apple;
  cd.componentFlags = 0;
  cd.componentFlagsMask = 0;
  
  result = NewAUGraph (outGraph);
  if (result != 0) return result;
  
  cd.componentType = kAudioUnitType_MusicDevice;
  cd.componentSubType = kAudioUnitSubType_DLSSynth;
  
  result = AUGraphAddNode (*outGraph, &cd, &synthNode);
  if (result != 0) return result;
    
  cd.componentType = kAudioUnitType_Effect;
  cd.componentSubType = kAudioUnitSubType_PeakLimiter;  
  
  result = AUGraphAddNode (*outGraph, &cd, &limiterNode);
  if (result != 0) return result;

  cd.componentType = kAudioUnitType_Output;
  cd.componentSubType = kAudioUnitSubType_DefaultOutput;  
  result = AUGraphAddNode (*outGraph, &cd, &outNode);
  if (result != 0) return result;
	
  result = AUGraphOpen (*outGraph);
  if (result != 0) return result;
	
  result = AUGraphConnectNodeInput (*outGraph, synthNode, 0, limiterNode, 0);
  if (result != 0) return result;

  result = AUGraphConnectNodeInput (*outGraph, limiterNode, 0, outNode, 0);
  if (result != 0) return result;

  result = AUGraphNodeInfo(*outGraph, synthNode, 0, outSynth);
  if (result != 0) return result;

  return result;
}

MIDIPortRef    gOutPort = 0;
MIDIEndpointRef  gDest = 0;
int        gChannel = 0;

void MidiOutShortMsg(unsigned char byte1,unsigned char byte2,unsigned char byte3,unsigned char byte4);


static void  MyReadProc(const MIDIPacketList *pktlist, void *refCon, void *connRefCon)
{
    if (gOutPort != 0 && gDest != 0)
    {
        MIDIPacket *packet = (MIDIPacket *)pktlist->packet;  // remove const (!)
        for (unsigned int j = 0; j < pktlist->numPackets; ++j)
        {
            for (int i = 0; i < packet->length; ++i)
            {
                //        printf("%02X ", packet->data<i>);
                
                // rechannelize status bytes
                if (((packet->data[i]) >= 0x80) && ((packet->data[i]) < 0xF0))
                    packet->data[i] = ((packet->data[i]) & 0xF0) | gChannel;
            }
            
            //      printf("\n");
            packet = MIDIPacketNext(packet);
        }
        
        if (packet->length<=4)
        MidiOutShortMsg(packet->data[0],packet->data[1],packet->data[2],packet->data[3]);
    }
}

static AudioUnit synthUnit = 0;
static AUGraph graph = 0;

static AudioUnit* p_synthUnit = &synthUnit;
static AUGraph* p_graph = &graph;

void MidiDelta(long delta)
{
  usleep(1000*delta);	
}


void MidiOutShortMsg(unsigned char byte1,unsigned char byte2,unsigned char byte3,unsigned char byte4)
{
  MusicDeviceMIDIEvent(synthUnit,byte1,byte2,byte3,byte4);
}

UInt32 MidiOutLongMsg(UInt8* Data, UInt32 DataLength)
{
    return MusicDeviceSysEx(synthUnit,Data,DataLength);
}

void MidiOutReset(char* MidiSelect)
{
  unsigned char i;
  unsigned char j;
////  char* bankPath="/Users/mogenslundholm/Music/Soundfonts/FluidR3_GM.sf2";
  
  CreateAUGraph(p_graph,p_synthUnit);
  // Set the sound font
  if (MidiSelect)
  {
      FSRef fsRef;
      FSPathMakeRef((const UInt8*)MidiSelect,&fsRef,0);
      AudioUnitSetProperty(synthUnit,kMusicDeviceProperty_SoundBankFSRef,kAudioUnitScope_Global,0,
                           &fsRef,sizeof(fsRef));
  }
    
    
  AUGraphInitialize(graph);
  CAShow(graph); 
  AUGraphStart(graph);
  for (j=0;j<16;j++)
    MusicDeviceMIDIEvent(synthUnit,0xB0+j,0,0,0);
  
  for (i=0;i<128;i++)
  for (j=0;j<16;j++)
    MusicDeviceMIDIEvent(synthUnit,0x80+j,i,0,0);
    
  // Midi Input
  MIDIClientRef client = 0;
  MIDIClientCreate(CFSTR("MIDI Echo"), NULL, NULL, &client);
   
  MIDIPortRef inPort = 0;
  MIDIInputPortCreate(client, CFSTR("Input port"), MyReadProc, NULL, &inPort);

}




