﻿//---------------------------------------------------------------------------
//
//  Program:     UnitOpenDialog.pas in MidiAndMusicXmlPlayer
//
//  Project:     MidiAndMusicXmlPlayer.dpr / MidiAndMusicXmlPlayer.lpi
//
//  Purpose:     Not used own Open-Dialog
//
//  Compilation: Compile with Delphi 4 (MidiAndMusicXmlPlayer.dpr) or
//               Lazarus (MidiAndMusicXmlPlayer.lpi)
//               or Delphi XE (MidiAndMusicXmlPlayer.dproj)
//
//  Description: Use the functions for TreeView to build the file-tree
//---------------------------------------------------------------------------

unit unitmidiopendialog;

{$IFDEF FPC}
{$mode delphi}
{$ENDIF}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, ComCtrls,
  UnitMidiDefinitions,UnitMessage;

// Some space around boxes and buttons
const GLUE = 5;

const
  SAnyMask = {$ifdef Darwin}'*'{$else}'*.*'{$endif};
  PathDelim = {$ifdef Darwin}'/'{$else}'\'{$endif};
  faSymLink = $00000040;

const
  // Select the icons used from the Icon bitmap
  // Note: 0 = yellow folder
  //       2 = yellow Win8 like
  IconFolder = 3;  // Blue folder
  IconFile = 1;    // The Midi/Music Xml icon

type

  { TFormMidiOpenDialog }

  TFormMidiOpenDialog = class(TForm)
    BevelTreeView: TBevel;
    EditFileName: TEdit;
    ImageButtonOkGrey: TImage;
    ImageButtonOkNew: TImage;
    ImageFolders: TImage;
    Timer: TTimer;
    TreeView: TTreeView;
    procedure ImageButtonOkNewClick(Sender: TObject);
    procedure OnCreate(Sender: TObject);
    procedure OnDblClick(Sender: TObject);
    procedure OnEdited(Sender: TObject; Node: TTreeNode; var S: string);
    procedure OnResize(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure TreeViewCustomDraw(Sender: TCustomTreeView; const ARect: TRect;
      var DefaultDraw: Boolean);
    procedure TreeViewDblClick(Sender: TObject);

  public
    { Public declarations }
    SelectedFileName: string;
    SetVisible: boolean;
    procedure FindFiles(Dir: string; RootNode: TTreeNode);
    function DirNotEmpty(Dir: string; RootNode: TTreeNode): boolean;
    function AddNode(Node: TTreeNode; Name: string): TTreeNode;
    function AddSubNode(Node: TTreeNode; Name: string;
                        IconNr: integer; Description: string): TTreeNode;
    function ShortPath(Name: string): string;
    procedure SavePosition;
  end;

var
  FormMidiOpenDialog: TFormMidiOpenDialog;

implementation

{$R *.lfm}

{ TFormMidiOpenDialog }

//---------------------------------------------------------------------------
//
//     Function:   FindFiles
//
//     Purpose:    To traverse the whole file structure and add the
//                 elements to the tree view.
//
//     Parameters: Dir = current directory
//                 RootNode = current node in the tree view
//
//     Returns:    void
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------

procedure TFormMidiOpenDialog.FindFiles(Dir: string; RootNode: TTreeNode);

var
  Info: TSearchRec;
////  FileName: string;
  DirName: string;
  InfoName: string;
  FileNameWithPath: AnsiString;
  Node: TTreeNode;
  MusicXmlFile: textfile;
  MidiFile: textfile;
  s1: string;
  s2: string;

  c1: char;
  c2: char;
  c3: char;
  c4: char;

  begin
  s1:='';
  s2:='';
{
  writeln('DefaultSystemCodePage:',DefaultSystemCodePage,
          ' DefaultFileSystemCodePage=',DefaultFileSystemCodePage,
          ' DefaultRTLFileSystemCodePage=',DefaultRTLFileSystemCodePage);
  writeln('LC_ALL=',GetEnvironmentVariable('LC_ALL'),' LC_MESSAGES=',
          GetEnvironmentVariable('LC_MESSAGES'),
          ' LANG=',GetEnvironmentVariable('LANG'));
}
  // Find sub-directories and recurse
  if FindFirst(Dir+SAnyMask,faDirectory,Info)=0 then
    try
      repeat
      if (Info.Attr and faDirectory<>0) and
         (Info.Name<>'.') and (Info.Name<>'..') and
         (Info.Attr and faSymLink=0) then
        begin // Dir - add name to directory path
        if DirNotEmpty(Dir+Info.Name+PathDelim,Node) then
        Node:=AddSubNode(RootNode,Info.Name,IconFolder,Dir+Info.Name);
        FindFiles(Dir+Info.Name+PathDelim,Node);
        end
      else
        begin // File - only midi and music xml.
        if ((Info.Name<>'.') and (Info.Name<>'..')) and
           (LowerCase(ExtractFileExt(Info.Name))='.mid') or
           (LowerCase(ExtractFileExt(Info.Name))='.xml') then
          begin
          if LowerCase(ExtractFileExt(Info.Name))='.xml' then
            begin
            {$i-}
            InfoName:=Info.Name;
            DirName:=Dir;
            AssignFile(MusicXmlFile,Dir+Info.Name);
            if IORESULT=0 then
            reset(MusicXmlFile);
            {$i+}
            if IORESULT=0 then
              begin
              // A music xml file has the word MusicXML in second line
              if not eof(MusicXmlFile) then readln(MusicXmlFile,s1);
              if not eof(MusicXmlFile) then readln(MusicXmlFile,s2);
              if (Pos('MusicXML',s1)>0) or (Pos('MusicXML',s2)>0) then
              Node:=AddSubNode(RootNode, Info.Name,IconFile,Dir+Info.Name);
              CloseFile(MusicXmlFile);
              end;
            end
          else
            begin // Midi File - Extension = ".mid"
            if FileExists(Dir+Info.Name) then
              begin
              AssignFile(MidiFile,Dir+Info.Name);
              if IORESULT=0 then
              reset(MidiFile);
              {$i+}
              if IORESULT=0 then
                begin
                // A midi file has the characters "MThd" in the start
                if not eof(MidiFile) then read(MidiFile,c1);
                if not eof(MidiFile) then read(MidiFile,c2);
                if not eof(MidiFile) then read(MidiFile,c3);
                if not eof(MidiFile) then read(MidiFile,c4);
                if (c1='M') and (c2='T') and (c3='h') and (c4='d') then
                Node:=AddSubNode(RootNode,Info.Name,IconFile,Dir+Info.Name);
                CloseFile(MidiFile);
                end;
              end;
            end;
          end;
        end;
      until FindNext(Info)<>0;
    finally
    FindClose(Info);
    end;
  end;

//---------------------------------------------------------------------------
//
//     Function:   DirNotEmpty
//
//     Purpose:    To traverse the whole file structure and test
//                 if directory is empty (i.e. contains no Midi nor Music Xml)
//
//     Parameters: Dir = current directory
//                 RootNode = current node in the tree view
//
//     Returns:    void
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------

function TFormMidiOpenDialog.DirNotEmpty(Dir: string; RootNode: TTreeNode): boolean;

var
  Info: TSearchRec;
  FileName: string;
  Node: TTreeNode;
  NotEmpty: boolean;
  MusicXmlFile: textfile;
  s1: string;
  s2: string;

  begin
  // Find sub-directories and call recurse
  if FindFirst(Dir+SAnyMask,faDirectory,Info)=0 then
    try
      NotEmpty:=false;
      repeat
      if (Info.Attr and faDirectory<>0) and
         (Info.Name<>'.') and (Info.Name<>'..') and
         (Info.Attr and faSymLink=0) then
        begin // Dir
        NotEmpty:=NotEmpty or DirNotEmpty(Dir+Info.Name+PathDelim,Node);
        end
      else
        begin // File
        if ((Info.Name<>'.') and (Info.Name<>'..')) and
           (LowerCase(ExtractFileExt(Info.Name))='.mid') or
           (LowerCase(ExtractFileExt(Info.Name))='.xml') then
          begin
          if LowerCase(ExtractFileExt(Info.Name))='.xml' then
            begin
            {$i-}
            AssignFile(MusicXmlFile,PChar(Dir+Info.Name));
            if IORESULT=0 then
            reset(MusicXmlFile);
            {$i+}
            if IORESULT=0 then
              begin
              if not eof(MusicXmlFile) then readln(MusicXmlFile,s1);
              if not eof(MusicXmlFile) then readln(MusicXmlFile,s2);
              if Pos('MusicXML',s1)>0 then NotEmpty:=true;
              if Pos('MusicXML',s2)>0 then NotEmpty:=true;
              CloseFile(MusicXmlFile);
              end;
            end
          else
            begin
            if FileExists(Dir+Info.Name) then NotEmpty:=true;
            end;
          end;
        end;
      until FindNext(Info)<>0;
    finally
    FindClose(Info);
    end;
    DirNotEmpty:=NotEmpty;
  end;




//---------------------------------------------------------------------------
//
//     Function:   AddNode
//
//     Purpose:    To add a node to the tree view. Show a folder icon.
//
//     Parameters: Node = the tree view node
//                 Name = text shown (music or download)
//
//     Returns:    void
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------

function TFormMidiOpenDialog.AddNode(Node: TTreeNode; Name: string): TTreeNode;
var
  NewNode: TTreeNode;

  begin
  NewNode:=TreeView.Items.Add(Node,Name);
  NewNode.ImageIndex:=IconFolder;
  NewNode.SelectedIndex:=IconFolder;
  NewNode.Data:=nil;
  AddNode:=NewNode;
  end;

//---------------------------------------------------------------------------
//
//     Function:   AddSubNode
//
//     Purpose:    To add a subnode to the tree view. Show a folder or
//                 file icon.
//
//     Parameters: Node = the tree view node
//                 Name = text shown (music or download)
//                 IconNr = The icon according to the icon file.
//                 (May be different on PC and MAC. MAC is p.t. blue)
//                 See definitions IconFolder and IconFile
//
//     Returns:    void
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------
function TFormMidiOpenDialog.AddSubNode(Node: TTreeNode; Name: string;
  IconNr: integer; Description: string): TTreeNode;

var
  RootNode: TTreeNode;
  PDescription: ^string;

  begin
  RootNode:=Node;
  Node:=TreeView.Items.AddChild(RootNode,Name);
  Node.ImageIndex:=IconNr;
  Node.SelectedIndex:=IconNr;
  New(PDescription);                 //// OBS: Add freeing the memory !!!!
  PDescription^:=Description;
  Node.Data:=PDescription;
  RootNode.Expand(true);
  AddSubNode:=Node;
  end;

//---------------------------------------------------------------------------
//
//     Function:   SavePosition
//
//     Purpose:    The save the position and size of the window
//                 in order to show it at same place next time
//
//     Parameters: <none>
//
//     Returns:    void
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------

procedure TFormMidiOpenDialog.SavePosition;
  var
    WindowPosition: TextFile;
    Home: string;

    begin
{$ifdef SaveAllOptions}
    Home:=GetEnvironmentVariable('HOME');
    if not DirectoryExists(Home+'/MidiOptions') then
      MkDir(Home+'/MidiOptions');
    AssignFile(WindowPosition,Home+'/MidiOptions/Default.txt');
    rewrite(WindowPosition);
    writeln(WindowPosition,Left);
    writeln(WindowPosition,Top);
    writeln(WindowPosition,Width);
    writeln(WindowPosition,Height);
    CloseFile(WindowPosition);
{$endif}
    end;

//---------------------------------------------------------------------------
//
//     Function:   ImageButtonOkClick
//
//     Purpose:    To handle the standard windows event ButtonOkClick
//                 By mouse click the the is selected and show in the edit
//                 field below the file tree view. The file is opened directly
//                 The system state is changed to be ready to start playing
//
//     Parameters: Sender = Windows standard
//
//     Returns:    void
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------

procedure TFormMidiOpenDialog.ImageButtonOkNewClick(Sender: TObject);

  begin
  Errors:=0;
  if FileExists(SelectedFileName) then
    begin
    FileName:=SelectedFileName;
    FormMessage.Initiate;
    SystemState:=MidiStarting;
    end
  else
    begin
    SystemState:=MidiNoFile;
    end;
  Visible:=false;
  SavePosition;
  end;



//---------------------------------------------------------------------------
//
//     Function:   OnCreate
//
//     Purpose:    To handle the standard windows event OnCreate
//                 If a file Default.txt exists then this file contains
//                 a position and size of this window. After this initialise
//                 the tree view of the directories for Music and Download
//
//     Parameters: Sender = Windows standard
//
//     Returns:    void
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------

procedure TFormMidiOpenDialog.OnCreate(Sender: TObject);

{$ifdef Darwin}
  begin
  end;

{$else}
var
  ImageList: TImageList;
  Home: string;
  WindowPosition: TextFile;
  L,T,W,H: integer;           // Local variables: Left, Top, Width, Height

  NodeMusicTop: TTreeNode;
  NodeDownloadTop: TTreeNode;
  NodeDesktopTop: TTreeNode;

  begin
  L:=Left;
  T:=Top;
  W:=Width;
  H:=Height;
  SelectedFileName:='';
  EditFileName.Text:='';
  Home:=GetEnvironmentVariable('HOME');
  if FileExists(Home+'/MidiOptions/Default.txt') then
    begin
    AssignFile(WindowPosition,Home+'/MidiOptions/Default.txt');
    reset(WindowPosition);
    if not eof(WindowPosition) then readln(WindowPosition,L);
    if not eof(WindowPosition) then readln(WindowPosition,T);
    if not eof(WindowPosition) then readln(WindowPosition,W);
    if not eof(WindowPosition) then readln(WindowPosition,H);
    CloseFile(WindowPosition);
    end;
  Left:=L;
  Top:=T;
  Width:=W;
  Height:=H;
  ImageList:=TImageList.Create(nil);
  ImageList.Clear;
  ImageList.Add(ImageFolders.Picture.Bitmap,ImageFolders.Picture.Bitmap);
  TreeView.Images:=ImageList;
  TreeView.Items.Clear;
  NodeMusicTop:=AddNode(nil,'Music');
  NodeDownloadTop:=AddNode(nil,'Download');
  NodeDesktopTop:=AddNode(nil,'Desktop');
  FindFiles(Home+'/Music/',NodeMusicTop);
  FindFiles(Home+'/Downloads/',NodeDownloadTop);
  FindFiles(Home+'/Desktop/',NodeDesktopTop);
  SelectedFileName:='';
  TreeView.FullExpand;
  end;
{$endif}


//---------------------------------------------------------------------------
//
//     Function:   OnDblClick
//
//     Purpose:    To handle the standard windows event OnDblClick
//                 The file is opened directly
//
//     Parameters: Sender = Windows standard
//
//     Returns:    void
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------

procedure TFormMidiOpenDialog.OnDblClick(Sender: TObject);

var
  Node: TTreeNode;
  PDescription: ^string;
  CloseAction: TCloseAction;

  begin
  Errors:=0;
  Node:=TreeView.Selected;
  if Node<>nil then
    begin
    PDescription:=Node.Data;
    if (PDescription<>nil) and (PDescription^<>'') and
       ((LowerCase(ExtractFileExt(PDescription^))='.mid') or
          (LowerCase(ExtractFileExt(PDescription^))='.xml')) then
      begin
      SelectedFileName:=PDescription^;
      EditFileName.Text:=ShortPath(PDescription^);
      ImageButtonOkNewClick(Sender);
      end
    else
      begin
      SelectedFileName:='';
      EditFileName.Text:='';
      end;
    end;
  SavePosition;
  end;

//---------------------------------------------------------------------------
//
//     Function:   OnEdited
//
//     Purpose:    To handle the standard windows event OnEdited
//                 A name in the tree view may be edited. Change
//                 the file name to the new one
//
//     Parameters: Sender = Windows standard
//
//     Returns:    void
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------

procedure TFormMidiOpenDialog.OnEdited(Sender: TObject; Node: TTreeNode;var S: string);

  var
    OldFileName: string;         //// description !!!!
    NewFileName: string;
    OldFilePath: string;
    NewFilePath: string;
    PDescription: ^string;
    p: integer; // Position of delimiter '/'
    i: integer; // Loop variable  for positions
    OldText: string;

    begin
    PDescription:=Node.Data;
    OldText:=Node.Text;
    // A file name - directory will have data = nil
    if PDescription<>nil then
      begin
      OldFilePath:=PDescription^;
      OldFileName:=ExtractFileName(OldFilePath);
      // Find the path part
      p:=0;
      for i:=1 to Length(OldFilePath) do if OldFilePath[i]='/' then p:=i;
      NewFilePath:=copy(OldFilePath,1,p);
      NewFileName:=S;
      NewFilePath:=NewFilePath+NewFileName;
      RenameFile(OldFilePath,NewFilePath);
      PDescription^:=NewFilePath;
      SelectedFileName:=PDescription^;
      EditFileName.Text:=ShortPath(PDescription^);
      end
    else
      begin
      // Can not be edited
      Node.Text:=Oldtext;
      S:=OldText;
      Node.Update;
      end;
    end;


//---------------------------------------------------------------------------
//
//     Function:   OnResize
//
//     Purpose:    To handle the standard windows event OnResize
//                 This window can be of any size and the elements
//                 are resized accordingly
//
//     Parameters: Sender = Windows standard
//
//     Returns:    void
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------

procedure TFormMidiOpenDialog.OnResize(Sender: TObject);

    begin
    VertScrollBar.Visible:=false;
    EditFileName.Top:=ClientHeight-(4*EditFileName.Height) div 2;
    EditFileName.Left:=2*GLUE;
    EditFileName.Width:=(7*ClientWidth) div 10;

    ImageButtonOkNew.Width:=(2*ClientWidth) div 10;
    ImageButtonOkNew.Top:=ClientHeight-(4*EditFileName.Height) div 2;
    ImageButtonOkNew.Left:=ClientWidth-((2*ClientWidth) div 10)-6*GLUE;
    ImageButtonOkNew.Height:=EditFileName.Height;

    ImageButtonOkGrey.Width:=ImageButtonOkNew.Width;
    ImageButtonOkGrey.Top:=ImageButtonOkNew.Top;
    ImageButtonOkGrey.Left:=ImageButtonOkNew.Left;
    ImageButtonOkGrey.Height:=ImageButtonOkNew.Height;

    TreeView.Top:=2*GLUE;
    TreeView.Height:=ImageButtonOkNew.Top-(ImageButtonOkNew.Height div 2)-3*GLUE;
    TreeView.Left:=2*GLUE;
    TreeView.Width:=ClientWidth-4*GLUE;

    BevelTreeView.Top:=Glue;
    BevelTreeView.Left:=Glue;
    BevelTreeView.Width:=ClientWidth-2*Glue;
    BevelTreeView.Height:=TreeView.Height+2*Glue;

    end;

//---------------------------------------------------------------------------
//
//     Function:   TimerTimer
//
//     Purpose:    To handle the standard windows event TimerTimer
//                 When a tree item is selected, there seem to be an
//                 updating problem - sometimes a wrong name appears
//                 (the previous selected item). Therefore the timer
//                 assures that the name will be right at next timer
//                 click.
//
//     Parameters: Sender = Windows standard
//
//     Returns:    void
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------

procedure TFormMidiOpenDialog.TimerTimer(Sender: TObject);
{$ifdef Darwin}
  begin
  end;

{$else}
var
  Node: TTreeNode;
  PDescription: ^string;

  begin
  Node:=TreeView.Selected;
  if Node<>nil then
    begin
    PDescription:=Node.Data;
    if (PDescription<>nil) and (PDescription^<>'') and
       ((LowerCase(ExtractFileExt(PDescription^))='.mid') or
          (LowerCase(ExtractFileExt(PDescription^))='.xml')) then
      begin
      SelectedFileName:=PDescription^;
      EditFileName.Text:=ShortPath(PDescription^);
      end;
    end;
  if SelectedFileName='' then
    begin
    ImageButtonOkNew.Visible:=false;
    ImageButtonOkGrey.Visible:=true;
    end
  else
    begin
    ImageButtonOkNew.Visible:=true;
    ImageButtonOkGrey.Visible:=false;
    end;
  end;
{$endif}

procedure TFormMidiOpenDialog.TreeViewCustomDraw(Sender: TCustomTreeView;
  const ARect: TRect; var DefaultDraw: Boolean);
begin

end;

procedure TFormMidiOpenDialog.TreeViewDblClick(Sender: TObject);
begin

end;


//---------------------------------------------------------------------------
//
//     Function:   ShortPath
//
//     Purpose:    In order to avoid the very long and strange
//                 names in the Apple's sandboxing environment
//                 only show 'Music' and 'Download' and the rest
//
//     Parameters: Name = the real file name with path
//
//     Returns:    A more readable name.
//
//     Notes:      <none>
//
//---------------------------------------------------------------------------

function TFormMidiOpenDialog.ShortPath(Name: string): string;

var
  Home: string;
  p: integer;

  begin
  Home:=GetEnvironmentVariable('HOME');
  p:=Pos(Home,Name);
  // Pass the HOME and '/'
  if p=1 then ShortPath:=copy(Name,Length(Home)+2,Length(Name)-Length(Home));
  end;




end.

